from __future__ import absolute_import
import platform
import os

if platform.system() == "Windows":
    libspath = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir, 'pyliblo3.libs'))
    if os.path.exists(libspath) and os.path.isdir(libspath):
        os.add_dll_directory(libspath)
    else:
        possible_paths = ['C:/Program Files/liblo/lib', 'C:/Program Files/liblo/bin']
        for path in ['C:/Program Files/liblo/lib', 'C:/Program Files/liblo/bin']:
            if os.path.exists(path) and os.path.isdir(path):
                os.add_dll_directory(path)
del platform
del os


from ._liblo import *



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# pyliblo - Python bindings for the liblo OSC library
#
# Copyright (C) 2007-2011  Dominic Sacré  <dominic.sacre@gmx.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import sys
import pyliblo3 as liblo


class DumpOSC:

    def blob_to_hex(self, b):
        return " ".join([ (hex(v/16).upper()[-1] + hex(v%16).upper()[-1]) for v in b ])

    def callback(self, path, args, types, src):
        write = sys.stdout.write
        ## print source
        #write("from " + src.get_url() + ": ")
        # print path
        write(path + " ,")
        # print typespec
        write(types)
        # loop through arguments and print them
        for a, t in zip(args, types):
            write(" ")
            if t == None:
                #unknown type
                write("[unknown type]")
            elif t == 'b':
                # it's a blob
                write("[" + self.blob_to_hex(a) + "]")
            else:
               # anything else
                write(str(a))
        write('\n')

    def __init__(self, port = None):
        # create server object
        try:
            self.server = liblo.Server(port)
        except liblo.ServerError as err:
            sys.exit(str(err))

        print("listening on URL: " + self.server.get_url())

        # register callback function for all messages
        self.server.add_method(None, None, self.callback)

    def run(self):
        # just loop and dispatch messages every 10ms
        while True:
            self.server.recv(10)


if __name__ == '__main__':
    # display help
    if len(sys.argv) == 1 or sys.argv[1] in ("-h", "--help"):
        sys.exit("Usage: " + sys.argv[0] + " port")

    # require one argument (port number)
    if len(sys.argv) < 2:
        sys.exit("please specify a port or URL")

    app = DumpOSC(sys.argv[1])
    try:
        app.run()
    except KeyboardInterrupt:
        del app




UDP = 1
UNIX = 2
TCP = 4

class Callback:
    def __init__(self, func: Callable, user_data: Any): ...


def time() -> float: ...


def send(target: Address | tuple[str, str] | int, str, *args) -> None: ...


class ServerError(Exception):
    def __init__(self, num: int, msg: str, where: str): ...




# decorator to register callbacks

class make_method:
    def __init__(self, path: str, types: str, user_data: Any = None): ...

    def __call__(self, f: Callable) -> Callable: ...

# common base class for both Server and ServerThread

class _ServerBase:

    def __init__(self, reg_methods: bool = True): ...

    def register_methods(self, obj: _ServerBase = None) -> None: ...

    def get_url(self) -> str: ...

    def get_port(self) -> int: ...

    def get_protocol(self) -> int: ...

    def fileno(self) -> int: ...

    def add_method(self, path: str, typespec: str, func: Callable, user_data: Any = None) -> None: ...

    def del_method(self, path: str, typespec: str | None = None) -> None: ...

    def add_bundle_handlers(self, start_handler: Callable, end_handler: Callable, user_data=None) -> None: ...

    def send(self, target: Address | tuple[str, int] | str, *args) -> None: ...

    @property
    def url(self) -> str: ...

    @property
    def port(self) -> int: ...

    @property
    def protocol(self) -> int: ...


class Server(_ServerBase):
    def __init__(self, port: int = None, proto: int | str = LO_DEFAULT, reg_methods: bool = True): ...
    def __dealloc__(self): ...
    def free(self) -> None: ...
    def recv(self, timeout: int | float | None = None) -> bool: ...

class ServerThread(_ServerBase):
    def __init__(self, port: int = None, proto: int | str = LO_DEFAULT, reg_methods: bool = True): ...
    def __dealloc__(self): ...
    def free(self) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...

class AddressError(Exception):
    def __init__(self, msg: str): ...

class Address:
    """
    An Address represents a destination for a message

    Possible forms:

    * `Address(hostname: str, port: int, proto: [int | str] = LO_UDP`)
    * `Address(port: int)`   # Assumes localhost
    * `Address(url: str)`    # A URL of the form 'osc.udp://hostname:1234/'

    Create a new `Address` object from the given hostname/port
    or URL.

    Args:
        hostname: the target's hostname - the name or an IP of the form '127.0.0.0'.
        port: the port number of the target
        proto: one of the constants `LO_UDP`, `LO_TCP`, `LO_UNIX` or a string like 'UDP', 'TCP' or 'UNIX'
        url: a URL in liblo notation, e.g. `'osc.udp://hostname:1234/'`.

    Raises:
        AddressError: if the given parameters do not represent a valid address.

    """

    cdef lo_address _address

    def __init__(self, addr: str | int, addr2: int = None, proto: int | str = None): ...
    def __dealloc__(self) -> None: ...
    def get_url(self) -> str: ...

    def get_hostname(self) -> str: ...
    def get_port(self) -> int | str: ...

    def get_protocol(self) -> int: ...

    @property
    def url(self) -> str: ...

    @property
    def hostname(self) -> str: ...

    @property
    def port(self) -> int: ...

    @property
    def protocol(self) -> int: ...


class Message:
    def __init__(self, path: str, *args): ...
    def add(self, *args: tuple[str, Any] | int | float | str | bool) -> None: ...


class Bundle:
    def __init__(self, *messages): ...
    def add(self, *args) -> None: ...



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# pyliblo - Python bindings for the liblo OSC library
#
# Copyright (C) 2007-2011  Dominic Sacré  <dominic.sacre@gmx.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import sys
import pyliblo3 as liblo


def make_message_auto(path, *args):
    msg = liblo.Message(path)

    for a in args:
        try: v = int(a)
        except ValueError:
            try: v = float(a)
            except ValueError:
                v = a
        msg.add(v)

    return msg


def make_message_manual(path, types, *args):
    if len(types) != len(args):
        sys.exit("length of type string doesn't match number of arguments")

    msg = liblo.Message(path)
    try:
        for a, t in zip(args, types):
            msg.add((t, a))
    except Exception as e:
        sys.exit(str(e))

    return msg


if __name__ == '__main__':
    # display help
    if len(sys.argv) == 1 or sys.argv[1] in ("-h", "--help"):
        sys.exit("Usage: " + sys.argv[0] + " port path [,types] [args...]")

    # require at least two arguments (target port/url and message path)
    if len(sys.argv) < 2:
        sys.exit("please specify a port or URL")
    if len(sys.argv) < 3:
        sys.exit("please specify a message path")

    if len(sys.argv) > 3 and sys.argv[3].startswith(','):
        msg = make_message_manual(sys.argv[2], sys.argv[3][1:], *sys.argv[4:])
    else:
        msg = make_message_auto(*sys.argv[2:])

    try:
        liblo.send(sys.argv[1], msg)
    except IOError as e:
        sys.exit(str(e))
    else:
        sys.exit(0)
